@@ -1,5 +1,5 @@
 # encoding: utf-8
-# Copyright (C) 2002-2013, Stefan Schwarzer <sschwarzer@sschwarzer.net>
+# Copyright (C) 2002-2014, Stefan Schwarzer <sschwarzer@sschwarzer.net>
 # See the file LICENSE for licensing terms.
 
 from __future__ import unicode_literals
@@ -7,10 +7,12 @@ from __future__ import unicode_literals
 import ftplib
 import itertools
 import os
+import pickle
 import posixpath
 import random
 import time
 import unittest
+import warnings
 
 import ftputil
 import ftputil.compat
@@ -379,12 +381,12 @@ class TestTimeShift(unittest.TestCase):
           (      0,           0),
           (      0.1,         0),
           (     -0.1,         0),
-          (   1500,           0),
-          (  -1500,           0),
-          (   1800,        3600),
-          (  -1800,       -3600),
-          (   2000,        3600),
-          (  -2000,       -3600),
+          (   1500,        1800),
+          (  -1500,       -1800),
+          (   1800,        1800),
+          (  -1800,       -1800),
+          (   2000,        1800),
+          (  -2000,       -1800),
           ( 5*3600-100,  5*3600),
           (-5*3600+100, -5*3600)]
         for time_shift, expected_time_shift in test_data:
@@ -405,23 +407,34 @@ class TestTimeShift(unittest.TestCase):
                           25*3600)
         self.assertRaises(ftputil.error.TimeShiftError, assert_time_shift,
                           -25*3600)
-        # Invalid time shift (too large deviation from full hours unacceptable)
+        # Invalid time shift (too large deviation from 15-minute units
+        # is unacceptable)
         self.assertRaises(ftputil.error.TimeShiftError, assert_time_shift,
-                          10*60)
+                          8*60)
         self.assertRaises(ftputil.error.TimeShiftError, assert_time_shift,
-                          -3600-10*60)
+                          -3600-8*60)
 
     def test_synchronize_times(self):
         """Test time synchronization with server."""
         host = test_base.ftp_host_factory(ftp_host_class=TimeShiftFTPHost,
                                           session_factory=TimeShiftMockSession)
-        # Valid time shift
-        host.path.set_mtime(time.time() + 3630)
-        host.synchronize_times()
-        self.assertEqual(host.time_shift(), 3600)
-        # Invalid time shift
-        host.path.set_mtime(time.time() + 3600+10*60)
-        self.assertRaises(ftputil.error.TimeShiftError, host.synchronize_times)
+        # Valid time shifts
+        test_data = [
+          (60*60+30,  60*60),
+          (60*60-100, 60*60),
+          (30*60+100, 30*60),
+          (45*60-100, 45*60),
+        ]
+        for measured_time_shift, expected_time_shift in test_data:
+            host.path.set_mtime(time.time() + measured_time_shift)
+            host.synchronize_times()
+            self.assertEqual(host.time_shift(), expected_time_shift)
+        # Invalid time shifts
+        measured_time_shifts = [60*60+8*60, 45*60-6*60]
+        for measured_time_shift in measured_time_shifts:
+            host.path.set_mtime(time.time() + measured_time_shift)
+            self.assertRaises(ftputil.error.TimeShiftError,
+                              host.synchronize_times)
 
     def test_synchronize_times_for_server_in_east(self):
         """Test for timestamp correction (see ticket #55)."""
@@ -567,6 +580,16 @@ class TestAcceptEitherUnicodeOrBytes(unittest.TestCase):
           self.host.walk, "/home/file_name_test/ä")
 
 
+class TestFailingPickling(unittest.TestCase):
+
+    def test_failing_pickling(self):
+        """Test if pickling (intentionally) isn't supported."""
+        with test_base.ftp_host_factory() as host:
+            self.assertRaises(TypeError, pickle.dumps, host)
+            with host.open("/home/sschwarzer/index.html") as file_obj:
+                self.assertRaises(TypeError, pickle.dumps, file_obj)
+
+
 if __name__ == "__main__":
     unittest.main()
     import __main__
